#Phylostats Functions
require(picante)
require(ape)
require(caper)
require(phytools)
require(ggtree)
require(compositions)
require(fields)
require(parallel)


#phylo.heatmap.loc creates a heatmap of the data along-side the phylogeny
# Ideally, use Liam Revell's version. U
phylo.heatmap.loc <- function(tree,Y,tipLabels=TRUE,...){
  if (all(colnames(Y) %in% tree$tip.label) && all(tree$tip.label %in% colnames(Y)) == FALSE){stop('colnames of Y and tiplabels of tree do not match')}
  edgs <- tree$edge
  n <- length(tree$tip.label)
  
  #extract order in which taxa show up on tree, from bottom to top
  ord <- match(edgs[,2],1:n)
  ord <- ord[is.na(ord)==F]
  
  
  if (n==dim(Y)[1]){Y <- t(Y)}
  m <- dim(Y)[1]

  layout(matrix(c(1,1,2),1,3))
  ### image of data ###
  image(1:m,1:n,Y[,ord],main='Data',xlab="sample",ylab='OTU')
  
  ### left-facing phylogeny with colored tip-labels corresponding to stacked plot above ###
  par(mar=c(5, 4, 4, 2)+.1)
  pp <- plot.phylo(tree,type='p', use.edge.length=F,direction='l',main='Community Phylogeny',show.tip.label=FALSE,...)
  if (tipLabels==TRUE){
    tiplabels(text= tree$tip.label,bg='white',col = 'black',cex=3)
  }
  ###########################
}


########## oldest.mrca from Liam Revell's blog #######
oldest.mrca<-function(tree,tips){
  H<-nodeHeights(tree)
  X<-mrca(tree)
  n<-length(tips)
  nodes<-height<-vector(); k<-1
  for(i in 1:(n-1)) for(j in (i+1):n){
    nodes[k]<-X[tips[i],tips[j]]
    height[k]<-H[match(nodes[k],tree$edge[,1]),1]
    k<-k+1
  }
  z<-match(min(height),height)
  return(nodes[z])
}


############ subtree ##############
subtree <- function(grp,tree){
  return(drop.tip(tree,setdiff(1:length(tree$tip.label),grp)))
}


###################### tree.ilr ############################################

tree.ilr <- function(phylo){
  #input phylogeny. Output will be a list containing (1) TreeILR basis, (2) list of Groups, (3) mapping from groups to ilr
  TreeILR = gsi.buildilrBase(gsi.merge2signary(as.hclust(phylo)$merge))
  
  Groups <- clade.members.list(phylo)
  
  GroupVector <- function(node,tree){
    #create vector in which the two descendant clades of a given node are +/-1, and all others are zero.
    #furthermore, all members of the same sub-clade will have the same sign
    node=as.numeric(node)
    ntips <- length(tree$tip.label)
    GrpVec <- rep(0,ntips)
    desc.nodes <- phangorn::Descendants(tree,node,type='children')
    for (i in 1:2){
      dum <- phangorn::Descendants(tree,desc.nodes[i],type='tips')[[1]]
      GrpVec[dum]=sign(i-1.1)
    }
    return(GrpVec)
  }

  GrpVec <- lapply(X=as.list(names(Groups)),FUN = GroupVector,tree=phylo)
  
  VecMatch <- function(Vec,ILR){
    if (is.list(Vec)){
      Vec=Vec[[1]]
      warning('Coercing Vec out of list using [[1]]')
    }
    isalleq<-function(X,y){isTRUE(all.equal(X,y))}
    ix=which(apply(X=sign(ILR),MARGIN = 2,FUN = isalleq,y=Vec))
    if (length(ix)==0){
      Vec=-Vec
      ix=which(apply(X=sign(ILR),MARGIN = 2,FUN = isalleq,y=Vec))
    }
    return(ix)
  }
    
  Grp2ilr <- lapply(X=GrpVec,FUN=VecMatch,ILR=TreeILR)
    
  
  GrpILR <- matrix(NA,ncol=2,nrow=ncol(TreeILR))
  GrpILR[,1] <- as.numeric(names(Groups))
  GrpILR[,2] <- unlist(Grp2ilr)
  colnames(GrpILR) <- c('Group','ILR index')
  
  output <- NULL
  output$TreeILR <- TreeILR
  output$Groups <- Groups
  output$Grp2ilr <- GrpILR
  return(output)
}


############################ subILR #####################################

subILR <- function(node,tree,Grp2ILR=NULL){
  #input node,treee and optional Grp2ILR matrix. 
  if (is.null(Grp2ILR)){
    Grp2ILR <- tree.ilr(tree)$Grp2ilr
  }
  
  dum <- phangorn::Descendants(tree,node,'all')
  dum <- dum[which(dum>length(tree$tip.label))]
  
  subILR <- Grp2ILR[which(Grp2ILR[,1] %in% c(node,dum)),2]
  
}


############################# tpval #####################################
tpval <- function(x){return(t.test(x)$p.value)}
ttst <- function(x){return(t.test(x)$statistic)}

############################# ilr.anova ##################################
ilr.anova <- function(R,treeILR){
  
  if(dim(R)[1]==dim(treeILR)[1]){
  warning('assuming the columns of R are parts and the rows are samples, and columns of treeILR are ilr basis elements')
  }
  a <- ilr(R,treeILR)
  k <- ncol(a)
  nn <- nrow(a)
  r <- c(t(a))
  f <- as.factor(1:k)
  tm <- gl(k,1,nn*k,f)
  av <- aov(r~tm)
  P <- summary(av)[[1]][1,5]
  
  ilraov <- NULL
  ilraov$aov <- av
  ilraov$P <- P

  return(ilraov)
}

############################# ilr.gen.anova ##################################
ilr.gen.anova <- function(R,treeILR,test='spherical'){
  # to be used 
  if(dim(R)[1]==dim(treeILR)[1]){
    warning('assuming the columns of R are parts and the rows are samples, and columns of treeILR are ilr basis elements')
  }
  a <- ilr(R,treeILR)
  p <- dim(a)[1]
  n <- dim(a)[2]
  ll <- lm(c(t(a))~rep(as.factor(1:n),p))
  av <- anova(ll,test=test)
  P <- av$`Pr(>F)`[1]
  
  ilraov <- NULL
  ilraov$aov <- av
  ilraov$P <- P
  
  return(ilraov)
}
# 
# ############################# getP #####################################
# getStats <- function(gg){
#     av <- aov(gg)
#     P <- summary(av)[[1]][1,5]
#     f <- summary(av)[[1]][1,4]
#     stats <- c(P,f)
#     names(stats) <- c('Pval','F')
#     return(stats)
# }

############################# ilr.sig ####################################
ilr.sig <- function(R,TreeILR,X=NULL,P=NULL,nclades=NULL,BonFerroni=TRUE,predict.data=FALSE,choice='t',GPU=FALSE){
  #Input compositional matrix, R, ilr basis matrix TreeILR, optional independent variable x, optional P-value cutoffs and number of clades to return
  #return is a list whose first element is a matrix giving the index and P-value for significantly non-zero ILR axes in R.
  # if x is input, the index and P-values will correspond come from a glm of the ilr coordinates on x. 
  # if x is a factor, the P-value will correspond to the F-statistic from an ANOVA. 
  # if choice = 't', the ilrs will be ranked by their test statistic from regression or zero-mean t-tests.
  # if choice = 'var', the ilrs will be ranked by the percent variance explained by their means/regressions on the original data
  #second element of list is the vector of relative abundances obtained using only these clades.
  # Note - default is to use BonFerroni correction to control family-wide error rate
  
  ILR_proj <- ilr(R,TreeILR)
  m=dim(ILR_proj)[1]
  D=dim(ILR_proj)[2]
  #now, we convert ILR_proj into a matrix - the class "rmult" caused problems with the split(a,col(a)) below
  ILR_proj <- matrix(c(ILR_proj),ncol= D)
  # note: to get R back, use 
  #ilrInv(ILR_proj,TreeILR)
  
  ############ extract P values and test statistics
  if (is.null(X)){ #standard t-test
    Pvals <- apply(ILR_proj,MARGIN = 2,FUN=tpval)
    ts <- abs(apply(ILR_proj,MARGIN = 2,FUN=ttst))
  } else {
    #### This function is highly parallelizable. This should be doable on a GPU
     gg <- lapply(split(ILR_proj,col(ILR_proj)), FUN = function(y,x){return(glm(y~x-1))},x=X)
     stats <- lapply(gg,getStats)
     stats <- matrix(unlist(stats),ncol=2,byrow=T)
     Pvals <- stats[,1]
     ts <- stats[,2]
  }
  
  ########### extract clades ###################
  if (choice %in% c('t','var') == FALSE){stop('improper choice input for ilr ranking')}
  if (choice=='t'){ # we choose clades based on their test-statistics
    
    ## pick sclades based on P-value threshold?
    if (is.null(P)){
      sclades=1:length(Pvals)
    } else {
      if (BonFerroni){
        sclades <- which(Pvals<=(P/ncol(ILR_proj))) #Bon-Ferroni correction - grabs all sig. clades
      } else {
        sclades <- which(Pvals<=P)
      }
    } 
    
    ## Now trim the scaldes - either pick nclades of them, or sort them based on test-statistic 
    if (is.null(nclades)==FALSE){ #user has input a desired number of clades. We'll grab the most significant ones
      
      if (length(sclades)<nclades){warning('number of significant clades is less than the nclades desired. Output contains all significant clades')
        } else {
        srt <- sort(ts[sclades],decreasing=T,index.return=T)
        sclades <- sclades[srt$ix[1:nclades]]
      }
    } else { 
        srt <- sort(ts[sclades],decreasing=T,index.return=T)
        sclades <- sclades[srt$ix]
    }
      
    } else { #We rank the ilrs based on the percent of variance explained - a solid regression of a rare species could account for little variation in our data.
      
      totvar <- var(c(ilr2clr(ILR_proj,V = TreeILR)))
      propvar <- rep(0,D)
      
      #### Calculate the coordinates, cfs, from the regressions/means
      if (is.null(X)){
        cfs <- colMeans(ILR_proj)
      } else {
        cfs <- matrix(rep(0,m*D),ncol=D)
      for (nn in 1:D){
        cfs[,nn] <- predict(gg[[nn]]) 
      }}
      
      #### iteratively compute total variance in dataset for each ilrs
      for (nn in 1:D){
        Is <- matrix(0,ncol=D,nrow=m)
        Is[,nn] <- cfs[,nn]
        propvar[nn] <- 1-var(c(ilr2clr(ILR_proj-Is,V=TreeILR)))/totvar
      }
      
      if (is.null(nclades)==FALSE){ #user has input a desired number of clades. We'll grab the most significant ones
          srt <- sort(propvar,decreasing=T,index.return=T)
          sclades <- srt$ix[1:nclades]
      } else { 
         srt <- sort(propvar,decreasing=T,index.return=T)
         sclades <- srt$ix
      }
    
  }
  
  ############# Impute data matrix with only the chosen clades ####################
  #note - we're taking the arithmetic means of the ILR balances to find the center in ILR coordinates
  gs <- NULL
  if (is.null(X)){
      mns <- rep(0,D)
      if (length(sclades)>1){
      mns[sclades] <- colMeans(ILR_proj[,sclades]) #these are the centers for the balances
      } else {
        mns[sclades] <- mean(ILR_proj[,sclades])
      }
      gs <- clo(t(exp(TreeILR %*% mns)))
      Rs <- ilrInv(matrix(rep(mns,m),ncol=D,byrow=T),TreeILR)
      coefs <- matrix(rep(mns[sclades],m),ncol=length(sclades),byrow=T)
  } else {
    # now we want to use our the glm for sclades to predict our data matrix, R.
    coefs <- matrix(rep(0,m*D),ncol=D)
    for (nn in 1:length(sclades)){
      coefs[,sclades[nn]] <- predict(gg[[sclades[nn]]]) 
    }
    Rs <- ilrInv(coefs,TreeILR)
    coefs <- coefs[,sclades]
  }
  
  Rs <- matrix(c(Rs),ncol=dim(Rs)[2])
  
  output <- NULL
  if (choice=='t'){
    output$ILRs <- matrix(c(sclades,Pvals[sclades]),ncol=2,byrow=FALSE)
    if (length(sclades)>1){
    colnames(output$ILRs) <- c('sig_ILRs','Pvals')
    } else {
      names(output$ILRs) <- c('sig_ILRs','Pvals')
    }
  } else {
    output$ILRs <- matrix(c(sclades,propvar[sclades]),ncol=2,byrow=FALSE)
    if (length(sclades)>1){
    colnames(output$ILRs) <- c('sig_ILRs','Prop.Var')
    } else {
      names(output$ILRs) <- c('sig_ILRs','Prop.Var')
    }
  }
  
  output$prediction <- Rs
  output$gs <- gs
  output$coefs <- coefs
  output$P.values <- Pvals
  return(output)
}

######################### ilr.vec ###############################
ilr.vec <- function(ix,n,V=NULL,GPU=FALSE){
  # Computes ilr basis element for primary partition. Input indexes, ix, which are partitioned.
  # If ix is a list of length 2, ilr basis element for finer partition will be created, separating
  # the group unlist(ix) into ix[[1]] and ix[[2]]
  # If you input an existing basis, V, the output will be a unit-norm vector orthogonal to the existing basis
  # If GPU=TRUE, this will be done using GPU-parallelization
  if (is.null(V)==FALSE){ if (min(dim(V))==(n-1)){stop('V is already a basis of the n-simplex')}}
  
  
  if (length(unlist(ix))==n || all(unlist(ix) %in% 1:n)==FALSE){stop('index set does not partition set 1:n')}
  if (is.list(ix) && length(ix)>2){stop('index list is longer than 2')}
  if (is.list(ix)==FALSE){
    ix <- list(ix)}
  ixGrp <- unlist(ix)
  
  ## First, if V is not null, we find the smallest group containing our index group
  ## Note - this is only used if ix 
  ################## INCOMPLETE - does not allow higher partitions ############################
  if (is.null(V)==FALSE){
    x = rep(0,n)
    x[ixGrp]=1
    y=x %*% V
    
    gg=1:n
      for (nn in 1:dim(V)[2]){
        gg=intersect(gg,which(sign(V[,nn])==sign(y[nn])))
      }
    ix[[2]]=setdiff(gg,ix[[1]])
  }

  
  if (length(ix)==1){ #primary partition
    ix = unlist(ix)
    r <- length(ix)
    s <- n-length(ix)
    a <- sqrt(s/(r*(r+s))) #ix elements
    b <- -sqrt(r/(s*(r+s))) #non-ix elements
    
    vec=rep(b,n)
    vec[ix]=a
  } else { #higher order partition
    ix1=unlist(ix[[1]])
    ix2=unlist(ix[[2]])
    r <- length(ix1)
    s <- length(ix2)
    a <- sqrt(s/(r*(r+s))) #ix1 elements
    b <- -sqrt(r/(s*(r+s))) #ix2 elements
    vec=rep(0,n)
    vec[ix1]=a
    vec[ix2]=b
  }
  return(vec)
}

#################################### find.unsplit.groups ###############################################
#useful function for grabbing groups given a partition

find.unsplit.Grps <- function(V){ #function to find out the groups which still need to be partitioned
  #input ilr sub-basis, V. output a list of unsplit groups.
  S <- sign(V)
  if (is.null(dim(S))){
    ix <- 1:length(S)
    m <- 1
    ss <- vector(mode='list',length=m+1)
    ss[1:2] <- split(ix,S)
  } else {
    ix=1:dim(S)[1]
    m <- dim(S)[2]
    ss <- vector(mode='list',length=m+1)
    ss[1:2] <- split(ix,S[,1])
  }
  
  
  if(m>1){
    for (ll in 2:m){
      ind <- which(S[,ll]!=0)
      inx <- which(unlist(lapply(ss,function(x,y){all(y %in% x && x %in% y)},y=ind)))
      dum <- split(ss[[inx]],S[which(S[,ll]!=0),ll])
      ss[[inx[1]]]=dum[[1]]
      ss[[ll+1]]=dum[[2]]
    }}
  ss <- ss[which(unlist(lapply(ss,function(x){return(length(x)>1)})))]
  return(ss)
}

##################################### ilr.complete ######################################################

ilr.complete <- function(V,tree,tips=TRUE){
  #input 'incomplete' ilr basis, V, and tree. This function produces one possible remaining ilr basis given the partitions defined in the tree
  if (is.null(dim(V))==FALSE){ #check that V matches the tree and is not already a basis for the full D-1 dimensional subspace 
    if (dim(V)[1]!=length(tree$tip.label)){stop('dimension of V does not match the number of tips on tree')}
    if (rankMM(V)==(dim(V)[1]-1)){stop('V is already of rank D-1')}
    if (all.equal(t(V) %*% V,diag(dim(V)[2]))!=TRUE){warning('V is not unitary')}
  } else {
    if(length(V)!=length(tree$tip.label)){stop('dimension of V does not match the number of tips on tree')}
  }
    
    D=length(tree$tip.label) #number of species
    S <- matrix(0,nrow=D,ncol=D-1)
    if (is.null(dim(V))){
      S[,1]<-sign(V)
      k=1                    #dimension of current ilr basis
    } else {
      S[,1:dim(V)[2]] <- sign(V)
      k=dim(V)[2]
    }
    rnk <- k
    ######################## internal functions #########################
    # find un-split groups
    find.unsplit.Grps(V)
    
    treesplit <- function(grp,tree){
      mrcn <- oldest.mrca(tree,grp)
      return(lapply(phangorn::Descendants(tree,phangorn::Descendants(tree,mrcn,type='children'),type='tips'),intersect,grp))
    }
    
    splitvec <- function(grp,tree,D){
      ts <- treesplit(grp,tree)
      vec <- rep(0,D)
      vec[ts[[1]]]=1
      vec[ts[[2]]]=-1
      return(vec)
    }
    ##########################################3#########################
    
    # iteratively split groups until we have full rank
    

    while (rnk < D-1){
        Groups <- find.unsplit.Grps(S[,1:k])
        ngs <- length(Groups)
        vecs <- lapply(Groups,splitvec,tree=tree,D=D)
        S[,1:(k+ngs)] <- matrix(c(S[,1:k],unlist(vecs)),nrow=D)
        k<-k+ngs
        rnk <- k
    }
    
    return(gsi.buildilrBase(S))
}
# 
# 
# ######################################## PhyloFactor #########################################
# PhyloFactor <- function(Rc,tree,X=NULL,nclades=NULL,P=NULL,stop.fcn = 'anova',choice='t',GPU=FALSE,...){
#   # Input:
#   # Rc - simplex-valued otu table hose columns are OTUs and rows are samples
#   # tree - phylogenetic tree. tip labels of tree must correspond to colnames of Rc
#   # X - optional independent variable. If included, ilr bases will be chosen based on regression of the ilr coefficients against X.
#   # nclades - optional number of partitions or factors to extract. If P is specified, default is to keep extracting factors until the ilr.anova of dataset yields a P-value greater than P. Otherwise, will extract a full basis of sequentially less important factors as defined by "test"
#   # P - optional P-value cutoff for ilr anova.
#   # independ.var - optional independent variable. If included, a glm on the coordinates of ilr basis candidates is used to determine the most important partitions/ilr-basis-candidates
#   # choice - takes values of "t" or "var". If "t", the best ilr basis elements will be those which maximize the t-statistic from either a test of zero-mean for the coordinate or, if input independ.var, from a glm.
#   #          If "var", the best ilr basis elements will be those which minimize the residual variance in the original dataset.
#   # (1) construct set of possible 1st-order partitions for each clade.
#   # (2) find partition with most significant effect
#   # (3) subtract out that parition's effect: apply(R,MARGIN=2,FUN=perturbe,y=gs^-1)
#   # (4) construct set of possible 2nd-order partitions...
#   # (5) repeat until (a) K partitions, (b) anova P value > Pstar...(c) other cutoffs (x% of variance explained, etc)
#   if (all(colnames(Rc) %in% tree$tip.label)==FALSE){stop('some columns of R are not in the tree')}
#   if (all(tree$tip.label %in% colnames(Rc))==FALSE){
#     tree <- drop.tip(tree,setdiff(tree$tip.label,colnames(Rc)))
#     warning('some tip labels of tree are not found in R. These tips are being dropped from this analysis')}
#   
#   if (is.null(P)==FALSE && (stop.fcn %in% c('KS','anova','clr.KS','ilr.KS')==FALSE)) {stop('If input P, then input stop.fcn must be either "KS" or "anova"')}
#   # First, we'll assemble the list of groups to include the tips
#   n=length(tree$tip.label)
#   Groups <- as.list(1:n)
#   clist <- clade.members.list(tree)
#   cc <- length(clist)
#   if (all(phangorn::Descendants(tree,n+1,type='children')>n)==FALSE){ 
#     #one of the children of the root node is a tip, already accounted for in our Groups=1:n above
#       Groups <- c(Groups,clade.members.list(tree)[3:cc])
#     } else {
#       Groups <- c(Groups,clist[2:cc])
#   }
#     names(Groups)[1:n]=1:n
#     
#     # Now we iterate and find the significant phyloFactors
#     PhyloFactors <- NULL
#     PhyloFactors$Groups <- Groups
#     
#     pfs=0
#     
#     clds <- Groups
#     # Then, we'll assemble the list of ilr bases
#     tILR <- matrix(unlist(lapply(clds,FUN = ilr.vec,n=length(tree$tip.label),V=PhyloFactors$ilrs,GPU=GPU)),ncol=length(clds),byrow=FALSE)
# 
#     if (is.null(P)){#user did not specify P-value threshold. setting 
#       PP=1
#       Ptree=.5
#     } else{
#       PP=P
#       Ptree=0
#       }
#     if (is.null(nclades)){#user did not specify a maximum number of clades. Will cluster all
#       nclades=n-1
#     }
#     
#     ################# the main loop ##########################
#     while(Ptree <= PP && pfs < nclades){
#       pfs=pfs+1
#     
#     # and a list of signifance for each bse
#       ilrTest <- ilr.sig(R=Rc,TreeILR=tILR,X = X,nclades=1,BonFerroni=FALSE,choice=choice,GPU=GPU)
#         if (dim(ilrTest$ILRs)[1]==0){break} #test came up with no significant taxa
#       best_cld <- ilrTest$ILRs[1]
#       GrpID <- which(names(Groups)==names(clds)[best_cld]) #Map our Groups to 
#       PhyloFactors$Nodes <- c(PhyloFactors$Nodes,names(clds)[best_cld])
#       PhyloFactors$ilrs <- matrix(c(PhyloFactors$ilrs,tILR[,best_cld]),ncol=pfs,byrow=FALSE)
#       
#       PhyloFactors$coefs <- matrix(c(PhyloFactors$coefs,ilrTest$coefs),ncol=pfs)
#       
#       #remove the winning clade
#       clds <- clds[setdiff(1:length(clds),best_cld)]
#       grps <- which(!lapply(clds,length)==1)
#       ## Now we remove the best_cld from all other grps - this ensures our next level's partition
#       ## does not include the clade we just regressed out. For instance, if we regressed out
#       ## the group (1,2) from (1,2,3,4,5), and there is also group (1,2,3), we want to turn it into just (3)
#       clds[grps]  <- lapply(clds[grps],FUN=function(x,y){if(all(y %in% x)){return(setdiff(y,x))} else {return(x)}},y=Groups[[GrpID]])
#       clds <- clds[!lapply(clds,length)==0] #remove null clades
#       
#       #Regress out the winning clade's impact
#       Rc <- clo(Rc/ilrTest$prediction)
#       
#       #PhyloFactors$R <- clo(t(exp(PhyloFactors$ilrs %*% as.matrix(PhyloFactors$coef))))
#       if (pfs==1){
#         PhyloFactors$ExplainedVar=ilrTest$ILRs[2]
#       } else {
#         PhyloFactors$ExplainedVar <- c(PhyloFactors$ExplainedVar,(1-sum(PhyloFactors$ExplainedVar[1:(pfs-1)]))*ilrTest$ILRs[2])
#       }
#       
#       if (pfs==(n-1)){break} else{
#       
#       #Assemble the next order of orthogonal bases
#       tILR <- matrix(unlist(lapply(clds,FUN = ilr.vec,n=length(tree$tip.label),V=PhyloFactors$ilrs,GPU=GPU)),ncol=length(clds),byrow=FALSE)
#       
#       
#       #test for significance in remaining subspace
#       # Test constructs remaining orthogonal ilr-space using ilr.complete, and then runs ilr.anova on
#       # the regressed-out data, Rc, projected onto the completed ilr-subspace
#       if (is.null(P)==FALSE){
#         if (stop.fcn == 'KS'){
#           Ptree <-  ks.test(ilrTest$P.values,runif(dim(tILR)[2]),...)$p.value
#         } else if (stop.fcn == 'anova'){
#           Ptree <- ilr.anova(Rc,ilr.complete(PhyloFactors$ilrs,tree)[,(pfs+1):(n-1)])$P #this should have the appropriate degrees of freedom.   
#         } else if (stop.fcn == 'clr.KS') {
#           Ptree <- ks.test(apply(clr(Rc),MARGIN=2,FUN = function(y,x){return(summary(aov(glm(y~x)))[[1]][1,'Pr(>F)'])},x=X),runif(n),...)$p.value
#         } else if (stop.fcn == 'ilr.KS'){
#           mat <- ilr(Rc,ilr.complete(PhyloFactors$ilrs,tree)[,(pfs+1):(n-1)])
#           Ptree <- ks.test(apply(mat,MARGIN=2,FUN = function(y,x){return(summary(aov(glm(y~x)))[[1]][1,'Pr(>F)'])},x=X),runif(dim(mat)[2]),...)$p.value
#         }
#         PhyloFactors$residual.P.values <- c(PhyloFactors$residual.P.values,Ptree)
#       }
#       }
#     }
#     
#     return(PhyloFactors)
# }
# 
# 
# ################################### predict.PhyloFactor #########################################
# predict.PhyloFactor <- function(PhyloFactors,tree,nfactors=1){
#   #input PhyloFactor object and its associated tree. Output will be data matrix formed by the nfactors most dominant factors
#   n=length(tree$tip.label)
#   mat <- matrix(rep(0,(n-1)*length(PhyloFactors$coefs[,1])),ncol=(n-1))
#   
#   ix = 1:nfactors
#   mat[,1:nfactors] = PhyloFactors$coefs[,1:nfactors]
#   Vc <- ilr.complete(V=PhyloFactors$ilrs[,1:nfactors],tree = tree)
#   Rp <- ilrInv(mat,Vc)
#   colnames(Rp) <- tree$tip.label
#   return(Rp)
# }
# 
# 
# 
# ################################### groups.Phylofactor ############################################
# groups.PhyloFactor <- function(PhyloFactors,tree,nfactors=1){
#   return(find.unsplit.Grps(PhyloFactors$ilrs[,1:nfactors]))
# }
# 
# atoms.PhyloFactor <- function(PF,nfactors=1){
#   return(atoms(PF$ilrs[,1:nfactors]))
# }
# 
# ################################### atoms #########################################################
# atoms <- function(V=NULL,G=NULL,set=NULL){
#   #input ILR sub-basis, V, or, alternatively, a list of groups G along with the set of all possible taxa, set
#   # G <- Groups[Groupn-n]
#   
#   if (is.null(V)==F){
#     G <- find.unsplit.Grps(V)
#     if (is.null(dim(V))){
#       set=length(V)
#     } else{
#     set=1:dim(V)[1]
#     }
#     dm <- setdiff(set,unique(unlist(G)))
#     return(c(G,dm))
#   } 
#   
#   if (is.null(G)==F && is.null(set)){
#     stop('If input only G, must also include the set of all possible elements')
#     } else {
#       
#       
#       #It's possible the G has some overlap in it, so first we need to split the groups in G
#       
#       ## This function returns the atomic sets in overlapping vectors, x and y.
#       ## Specifically, it returns the elements unique to x, unique to y, and the intersect of x,y 
#       set.split <- function(x,y){
#         if (length(intersect(x,y))==0){
#           return(NULL)
#         } else {
#           C <- setdiff(union(x,y),intersect(x,y))
#           return(unique(list(setdiff(x,C),setdiff(y,C),C)))
#         }
#       }
#       
#       ## the next bit of code iterates through the list of groups and, if there is overlap, splits the sets into atomic elements
#       for (nn in 1:(length(G)-1)){
#         for (mm in (nn+1):length(G)){
#           if (any(G[[nn]] %in% G[[mm]])){
#             dum <- set.split(G[[nn]],G[[mm]])
#             if (length(dum)==2){
#               G[c(nn,mm)] <- dum
#             } else {
#               G[c(nn,mm)] <- dum[1:2]
#               G <- c(G,dum[3])
#             }
#           }
#         }
#       }
#       
#       }
#       
#       dm <- list(setdiff(set,unique(unlist(G))))
#       return(c(G,dm))
#       ### Now we need to iterate through our group and split those groups with overlap
#   }
# 
# 
# 
# ################################### colorGroups ###################################################
# ColorGroups <- function(tree,grps,color.scheme=rainbow){
#   nedges <- Nedge(tree)
#   cols <- rep('Black',nedges)
#   ng <- length(grps)
#   for (nn in 1:(ng-1)){
#     taxa <- match(as.numeric(tree$tip.label),grps[[nn]])
#     taxa <- taxa[is.na(taxa)==F]
#     cols[extractEdges(tree,taxa = as.character(grps[[nn]]),type=3)] = color.scheme(ng)[nn]
#   }
#   return(cols)
# }
# 
# ##################################### matchNodes #################################################
matchNodes <- function(tree,PhyloFactors,nfactors=1){
  grps = NULL
  Nodes <- length(tree$tip.label)+1
  if (nfactors>1){
  for (nn in 2:nfactors){
    Nodes <- c(Nodes,findMRCA(tree,which(PhyloFactors$ilrs[,nn]!=0)))
  }
  }
  return(Nodes)
}

##################################### ILRtoNode ##################################################
ILRtoNode <- function(tree,V){
  txa <- which(V!=0)
  return(max(Reduce(intersect,phangorn::Ancestors(tree,txa))))
}



##################################### plot.PhyloFactors ############################################
plot.PhyloFactor <- function(PhyloFactors,tree,nfactors,nodeSize=3,...){
  #Now, we're going to re-root the tree according to the first partition
  tr <- reroot(tree,node = as.numeric(PhyloFactors$Nodes[1]))
  grps <- groups.PhyloFactor(PhyloFactors,tr,nfactors)
  R.est <- predict.PhyloFactor(PhyloFactors,tr,nfactors=2)[,match(tr$tip.label,tree$tip.label)]
  grpcols <- ColorGroups(tr,grps)
  ewd <- rep(3,length(grpcols))
  
  phylo.heatmap.loc(tr,R.est,tipLabels = TRUE,edge.color=grpcols,edge.width=ewd,...)
  
  Nodes <- matchNodes(tr,PhyloFactors,nfactors) #match nodes of PhyloFactors to our newly rooted tree, tr
  nodelabels(as.character(1:nfactors),node=Nodes,cex=nodeSize)
}



####################################### PercentCorrect ############################################
PercentCorrect <- function(effect,sample.size,reps,tree,method='ILR',TreeV=NULL,Groups=NULL,nclades=1,choice='t'){
  #input effect size, n, and method for regression in c("ILR", "PhyloFactor")
  #output the percent of regressions which correctly ID'd the clade of interest
  
  if (method %in% c('ILR','PhyloFactor') == FALSE){stop('unknown input "method" ')}
  if (method != 'PhyloFactor'){if (is.null(TreeV)){TreeV <- tree.ilr(tree)$TreeILR}}
  n <- length(tree$tip.label)
  if(is.null(Groups)){Groups <- clade.members.list(tree)}
  
  ILR.to.node <- apply(TreeV,MARGIN = 2,FUN=function(v,tree){return(ILRtoNode(tree,v))},tree=tree)
  perc.correct = rep(0,length(sample.size))
  val <- 0
  
  if (nclades>1 && length(effect)!=nclades){
    warning('input "effect" is not of length nclades - choosing random effects as lnormal random variables')
    effect <- rlnorm(nclades)
  }
  
  for (p in sample.size){
    val = val+1
    CorrectClade <- rep(0,reps)
    X <- as.factor(c(rep(1,p),rep(2,p)))
  for (rr in 1:reps){
    Groupn <- sample((n+2):(2*n-1),nclades)  #Groups[[1]] is all of the taxa in the tree - hence, it's excluded to ensure one sub-tree responds differently to X.
    
    Abundances <- matrix(round(rlnorm(2*p*n,meanlog=8,sdlog=.5)),nrow=2*p)
    sigClades <- Groups[Groupn-n] #significant clades
    for (cc in 1:nclades){
    Abundances[1:p,sigClades[[cc]]] <- Abundances[1:p,sigClades[[cc]]]*effect[cc] #The effects of significant Clades
    }
    Rc <- clo(Abundances)
    colnames(Rc) <- tree$tip.label
    Clade <- rep(NA,nclades)
      if (method=='PhyloFactor'){
        PF <- PhyloFactor(Rc=Rc,tree=tree,X=X,nfactors=nclades,choice=choice)
        # Clade <- as.numeric(phyFacts$Nodes)
        Fclades <- lapply(atoms(PF$basis),sort)
        Tclades <- lapply(atoms(G=sigClades,set=1:50),sort)
        # CorrectClade[rr] = sum(Clade %in% Groupn)/nclades
        correct <- sum(Fclades %in% Tclades)
        if (correct==(nclades+1)){
          CorrectClade[rr] = 1
        } else{
          CorrectClade[rr] = correct/nclades}
      } else {
        
        for (cc in 1:nclades){
        rootedILR <- ilr(Rc,TreeV)
        pvals <- apply(rootedILR,MARGIN=2,FUN=function(y,x){return(summary(aov(y~x))[[1]][1,'Pr(>F)'])},x=X)
        # Map the most significant ILR coordinate to node in tree
        nn=which(pvals==min(pvals))
        Clade[cc] <- ILR.to.node[nn]
        
        #regress out the effect of dominant ILR
        dum <- matrix(0,dim(rootedILR)[1],dim(rootedILR)[2])
        dum[,nn] <- predict(glm(rootedILR[,nn]~X))
        Rc <- clrInv(clr(Rc)- clr(ilrInv(dum,V=TreeV)))
        }
        #the "right Answer" for the tILR would be the 
        rightAns <- unlist(lapply(as.list(Groupn),function(grp,tree,x){return(nodepath(tree,grp,x)[2])},tree=tree,x=n+1))
        CorrectClade[rr] = sum(Clade %in% rightAns)/nclades
      }
  }
  
  perc.correct[val] <- sum(CorrectClade)/length(CorrectClade)
  }
  return(perc.correct)
}

# 
# ############ 3/20/16 - read and call this function - it's in PhyloStats for future use
# phyloFcluster <- function(ncores=2,...){
#   #creates cluster with all the necessary packages and functions to use these functions in parallel
#   cl <- makeCluster(ncores,...)
#   clusterEvalQ(cl,library(picante))
#   clusterEvalQ(cl,library(ape))
#   clusterEvalQ(cl,library(caper))
#   clusterEvalQ(cl,library(phytools))
#   clusterEvalQ(cl,library(ggtree))
#   clusterEvalQ(cl,library(phangorn))
#   clusterEvalQ(cl,library(compositions))
#   clusterEvalQ(cl,library(stats))
#   clusterEvalQ(cl,source('C:/Users/Big Alculus/Documents/Boulder/PhyloStats/R files/PhyloStats.R'))
#   return(cl)
# }
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# ######### Logratio - useful function for compositional data analysis ##########
# logratio <- function(x,y=1-x){log(x/y)}
# 
# ### 
# 
# ### Individual clade t.test or ANOVA
# clade.test <- function(cld,tree,Y,x){
#   #input tree, OTU table Y, and categorical variable x.If x contains two factors, this will perform a two-
#   #sample t-test. If x contains more than 2 factors, this function performs an anova.
#   
#   if (length(setdiff(rownames(Y),tree$tip.label))==0){Y<-t(Y)}
#   #The rows of Y are samples, the columns are species.
#   
#   ingrp <- rowSums(Y[,cld])/length(cld)
#   types <- unique(x)
#   Data <- data.frame(x,ingrp)
#   if (length(types)==2){
#     return(t.test(logratio(Data[Data$x==types[1],2]),logratio(Data[Data$x==types[2],2])))
#   }
#   else{
#     return(aov(logratio(ingrp)~x,data=Data))
#   }
# }
# 
# 
# 
# ### Clade GLM - produces GLM fit of log-ratios for a particular clade
# clade.glm <- function(cld,tree,X,Dep){
#   if (length(setdiff(rownames(X),tree$tip.label))==0){X<-t(X)}
#   ingrp <- rowSums(X[,cld])/length(cld)
#   gg<-lm(logratio(ingrp)~factor(Dep))
#   return(gg)
# }
# 
# getPvalue <- function(av){summary(av)[[1]]["Pr(>F)"][[1]][1]}
# 
# 
# ### Tree test - tests every clade for significance using cld.test
# tree.test <- function(tree,Y,x){
#   if (length(setdiff(rownames(Y),tree$tip.label))==0){Y<-t(Y)}
#   Groups <- clade.members.list(tree)
#   test.list<-lapply(Groups,clade.test,tree=hp,Y<-Y,x<-x)
#   Pvalues <- as.numeric(lapply(test.list,getPvalue))
#   Pvalues <- data.frame(names(Groups),Pvalues)
#   colnames(Pvalues)<-c('Group Number','Pvalue')
#   output <- list(test.list,Pvalues)
#   names(output)<-c('Sig.Tests','Pvalues')
#   return(output)
#   
# }


